library IEEE;
use  IEEE.STD_LOGIC_1164.all;
use  IEEE.STD_LOGIC_ARITH.all;
use  IEEE.STD_LOGIC_UNSIGNED.all;

entity banker is
  generic (n         :integer :=3);
  port (Clk          : in std_logic;
        Reset        : in std_logic;
        playerId     : in std_logic_vector (n-1 downto 0); -- from
        Action       : in std_logic_vector (2*n-3 downto 0); --from scorer 
        TakeAction   : in std_logic; 
        cgNewHand    : in std_logic;         
        bet          : in std_logic_vector (11 downto 0);
        
        balance      : out std_logic_vector (11 downto 0); --Max1024
        GameOver     : out std_logic; -- The whole game is over
        balancerdy   : out std_logic);

  
end banker;

architecture a of banker is
  
  type matrix is array (integer range n-2 downto 0) of std_logic_vector (11 downto 0);
  signal  initialBank    : std_logic_vector (11 downto 0);
  signal  CheckPlayer    : std_logic_vector (n-2 downto 0);
  signal  End_Game       : std_logic; -- end of hand
  signal  HP_Stop        : std_logic;
  signal  bet_store      : matrix;
--signal  G              : STD_LOGIC;
  signal  Count          : std_logic_vector (6 downto 0);
  signal  End_BJ         : std_logic;
  signal  PL_Out         : std_logic_vector (n-2 downto 0);
  signal  bank           : matrix;
  signal  betstore       : matrix;
  signal  BetFlag        : std_logic;
  signal  EndHand        : std_logic;


begin


---------------------------
-- Handling Total Scores --
---------------------------
-- player1
  process (Clk, Reset, bet, TakeAction, cgNewHand,PlayerId)
    variable ActionDone: std_logic;
  begin
    
    if (Clk = '1' and Clk'event) then
      if (Reset = '1') then
        initialBank <= "000001111111";
        for i in n-2 downto 0 loop
          bank(i) <= initialBank;
        end loop;
      else
        for i in n-2 downto 0 loop
          if PlayerId(i) = '1' then
            balance <= bank(i);
          end if;
        end loop;
        if (cgNewHand = '1') then 
          BetFlag <= '1';
        end if;
      end if;
      if BetFlag ='1' then
        balancerdy <= '0';              -- added by JDW
        for i in n-2 downto 0 loop
          if PlayerId(i) = '1' then
            betstore(i) <= bet;
            balance <= bank(i);
          end if;
        end loop;
        if PlayerId(n-2) = '1' then
          BetFlag <= '0';
        end if;
      end if;	

      if EndHand = '1' then
--        balancerdy <= '0';    -- removed by JDW
        EndHand <= '0';
      end if;        
      if (TakeAction = '1') then  
        if (ActionDone = '0') then                       
          for i in n-2 downto 0 loop
            --case when player = Win
            if Action(2*i + 1 downto 2*i) = "11" then  -- blackjack
              if (bank(i) >= betstore(i)) then    
                bank(i) <= ( betstore(i) + betstore(i)) + bank(i);
              else
                bank(i) <= ( bank(i) + bank(i)) + bank(i);
              end if;		
              --case when player wins (no BJ)
            elsif 
              Action(2*i + 1 downto 2*i) = "10"    then -- win
              if (bank(i) >= betstore(i)) then    
                bank(i) <= ( betstore(i)) + bank(i);
              else
                bank(i) <=  bank(i) + bank(i);
                
                --case when player = push (Tie)
              end if;
            elsif 
              Action(2*i + 1 downto 2*i) = "01" then -- tie
              bank(i) <= bank(i);			
              
              --case when player = Lose
            elsif 
              Action(2*i + 1 downto 2*i) = "00" then -- lose
              if (bank(i) >= betstore(i)) then                     
                bank(i) <= bank(i) - betstore(i);
              else
                bank(i) <= "000000000000";
                PL_Out(i) <= '1';
              end if;
            end if;
          end loop;
          balancerdy <= '1';
          EndHand <= '1';
          ActionDone := '1';
        end if;
      else
        ActionDone := '0';
      end if;
    end if;
    
  end process;




-------------------------------------------------------------------
----------------------------------------------
-- Set End_Game to high after each end game --
----------------------------------------------
  process(Clk, Reset)
  begin
    if (Reset = '1') then
      End_Game <= '0';
    elsif (Clk'event and Clk = '1') then
      if (EndHand = '1') then
--balancerdy <= '0';
--EndHand <= '0';
        End_Game <= '1';
      else End_Game <= '0';
      end if;
    end if;
  end process;

-- OUTPUT Game_Over for the Controller --

  process(Clk, Reset, End_BJ)
  begin
    if (Reset = '1') then
      GameOver <= '0';
    else
      -- elsif (Clk'EVENT and Clk = '1') then
      if (End_BJ = '1') then
        GameOver <= '1';
      else GameOver <= '0';
      end if;
    end if;
  end process;


-----------------------------------------------------------------
-- End after 100 Games or after all Players finish their Chips --
-- or any player has 1024 chips
-----------------------------------------------------------------
  process (Clk, Reset, End_Game)
  begin
    if (Reset = '1') then
      Count <= "0000000";
    elsif (Clk'event and Clk = '1') then
      if (End_Game = '1') then
        Count <= Count + "0000001";
      else Count <= Count;
      end if;
    end if;
  end process;

  process (Clk, Reset, PL_Out, Count)
    variable MaxB :std_logic;
    variable Plcheck: integer;
  begin
    if (Reset = '1') then
      End_BJ <= '0';
    elsif (Clk'event and Clk = '1') then
      bankmax : for i in n-2 downto 0 loop
        if bank(i) >= "001111111111" then
          MaxB := '1';
          exit bankmax;
        end if;
      end loop bankmax;

      -- for i in n-2 downto 0 loop
      for i in n-2 downto 0 loop
        if PL_Out(i) = '1' then
          Plcheck := PLcheck + 1;
        end if;
      end loop;		
      
      if (Count = "1100100") or (Plcheck = n-2) or (MaxB='1') then
        MaxB := '0';
        End_BJ <= '1';
        Plcheck := 0;
      else End_BJ <= '0';
      end if;
      -- end loop;
    end if;

  end process;

end a;
