library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_arith.all;
use IEEE.std_logic_unsigned.all;

entity Bjsk is
-- Generic parameters
  generic (n : integer := 3);                  -- Number of players
  port(
    Clk   : in std_logic;                      -- Global Clk
    Reset : in std_logic;                      -- Global Reset
-- from Card Generator 
    Card  : in std_logic_vector (3 downto 0);  -- Card Value

-- form controller
    PlayerId : in std_logic_vector (n-1 downto 0);
    cgIssue  : in std_logic;            -- Card has been issued
    --  cgNewHand       : IN STD_LOGIC; -- New hand
    PlStand  : in std_logic;  --- To inform that all players are standing
--from banker
    GameOver : in std_logic;

-- General Outputs      
    Action     : out std_logic_vector (2*n-3 downto 0);  -- Output to the Banker
    TakeAction : out std_logic;         -- To the banker to start calculating
    skBust     : out std_logic  -- To the controller to infrm about a player bust
---Temp outputs
--       HPTotal        : OUT STD_LOGIC_VECTOR (4 downto 0); 
--       Card1         : OUT STD_LOGIC_VECTOR (4 downto 0)    
    );

end Bjsk;


architecture behavior of Bjsk is

  type matrix is array (integer range n-1 downto 0) of std_logic_vector(4 downto 0);
  type matrix1 is array (integer range n-1 downto 0) of std_logic_vector(2 downto 0);
--type matrix2 is array (integer range n-1 downto 0 ) of STD_LOGIC_VECTOR(2 downto 0);

--signal CardCount : STD_LOGIC_VECTOR (n-1 downto 0) := (others => '0');
  signal cardIssued   : std_logic;
  signal Optotal      : matrix;         -- To keep track of total card values
  signal BJflagmatrix : matrix1;  -- To check numbe of cards for Blackjack
--signal PlayerPrev : matrix2 := (others => "000"); 
--signal PlayerStand : STD_LOGIC_VECTOR (n-1 downto 0); 
--constant CheckStand : STD_LOGIC_VECTOR (n-1 downto 0) := (others => '1');
--signal AceMatrix   : matrix1;

--signal FlagBJ : STD_LOGIC;
  signal LastPlayer  : std_logic;
--signal NewHand : STD_LOGIC;
  signal HP_BJ       : std_logic;  -- To check if the houseplayer has a blackjack
  signal InitOptotal : std_logic;
  signal skBustV : std_logic_vector(n downto 0) := (others => '0');
--signal ProcInitOptotal: STD_LOGIC;

begin
-- Card Issued
-- if card is issued then check playerId and calculate the total card value
  process (Reset, GameOver, PlayerId, cardIssued, Clk, Card, cgIssue, PlStand)
    variable CardValue    : std_logic_vector (4 downto 0);
    variable Acematrix    : matrix1;
    variable TempOptotal  : std_logic_vector (4 downto 0);
    variable IssueChecked : std_logic;
    variable PlayerDiff   : std_logic_vector (4 downto 0);
    variable HPDiff       : std_logic_vector (4 downto 0);
    variable zeroV : std_logic_vector(n downto 0) := (others => '0');
  begin
    if ((Reset = '1') or (GameOver = '1')) then  --- Reset all signals and start over
      for i in n-1 downto 0 loop
        Optotal(i)      <= "00000";
        BJflagmatrix(i) <= "000";
        AceMatrix(i)    := "000";
      end loop;
    else


      if (Clk = '1' and Clk'event) then
        if (skBustV and PlayerId) = zeroV then
          skBust <= '0';
        else
          skBust <= '1';
        end if;

        if (InitOptotal = '1') then
          Optotal      <= (others => "00000");
          BJflagmatrix <= (others => "000");
          Acematrix    := (others => "000");
          InitOptotal  <= '0';
          TakeAction   <= '0';
        end if;
        if (cgIssue = '1') then         -- If card is issued
          if (IssueChecked = '0') then  -- So that Operations performed only once after cgIssue goes high irrespective of the number of clock cycles 
            for i in n-1 downto 0 loop  --To check which player has the card issued
              if PlayerId(i) = '1' then
                CardValue             := "00000";
                CardValue(3 downto 0) := Card;
                --              Card1 <= CardValue;
                if (CardValue = "00001") then  -- To check if the card is an Ace which is a special condition
                  Optotal(i) <= Optotal(i) + "01011";
                  if (BJflagmatrix(i) < "011") then
                    BJflagmatrix(i) <= BJflagmatrix(i) + "001";  -- increment card count
                  end if;
                  if Optotal(i) > "10101" then
                    Optotal(i) <= Optotal(i) - "01011" + "00001";
                  else
                    Acematrix(i) := Acematrix(i) + "001";  -- If value of 11 is taken keep track for later use
                  end if;
                else
                  if (BJflagmatrix(i) < "011") then
                    BJflagmatrix(i) <= BJflagmatrix(i) + "001";
                  end if;
                  TempOptotal := Optotal(i) + CardValue;
                  Optotal(i)  <= Optotal(i) + CardValue;
                  if (TempOptotal > "10101") then  -- to check if the total card value is greater than 21
                    Aces : for j in 7 downto 0 loop
                      if (Acematrix(i) > "000") then  -- To check if the player has any Aces whose value can be subtracted
                        TempOptotal    := TempOptotal - "01010";
                        Acematrix(i)   := Acematrix (i) - "001";
                        if TempOptotal <= "10101" then
                          exit Aces;
                        end if;
                      else
                        exit Aces;
                      end if;
                    end loop Aces;
                  end if;
                  if (TempOptotal > "10101") then  --If total greater than 21 send signal to the controller that player is bust
                    skBustV(i) <= '1';
                  end if;
                  Optotal(i) <= TempOptotal;
                end if;

                if (i = n - 1)then
                  LastPlayer <= '1';
                end if;
              end if;
            end loop;
            IssueChecked := '1';
          end if;
          -- This part is commented for later in case all players stand
          --for i in n-1 downto 0 loop
          --    if (BJflagmatrix(i) > PlayerPrev(i)) then
          --            PlayerStand(i) = '1';
          --    end if;
          --end loop;
          --if PlayerStand = Checkstand then
          --    Calculate <= '1';
          --    CountPlayer <= (OTHERS=>'0');
          --else 
          --    CountPlayer <= (OTHERS=>'0');
          --end if;
          --PlayerPrev <= BJflagmatrix;
        else
          IssueChecked := '0';
          cardIssued   <= '0';
          --if (InitOptotal = '1') then
          --    Optotal <= (Others => "00000");
          --    BJflagmatrix <= (Others=>"00");
          --    ProcInitOptotal<='1';
          --end if;                             
        end if;

--when all players stand and controller waits for balance ready to be set high by the banker

        if (PlStand = '1') then  -- to check if the hand is over and scores can be calculated
          skBustV <= (others => '0');
          if (Optotal(n-1) = "10101")then  --to check if dealer has a blackjack
            if (BJflagmatrix(n-1) = "010") then
              HP_BJ <= '1';
            end if;
          end if;

          for i in n-2 downto 0 loop
            if (Optotal(i) = "10101") then
              if (BJflagmatrix (i) = "010") then  -- if the player has a blackjack
                if (HP_BJ = '1') then
                  Action(2*i+1 downto 2*i) <= "01";
                else
                  Action(2*i+1 downto 2*i) <= "11";
                end if;
              else
                if (HP_BJ = '1') then
                  Action(2*i+1 downto 2*i) <= "00";
                else
                  Action(2*i+1 downto 2*i) <= "10";
                end if;
              end if;
            else
              if (HP_BJ = '1') then
                Action(2*i+1 downto 2*i) <= "00";
              else
                if (Optotal(i) > "10101") then  -- if player has card value of greater than 21
                  if (Optotal(n-1) > "10101") then
                    Action(2*i+1 downto 2*i) <= "01";
                  else
                    Action(2*i+1 downto 2*i) <= "00";
                  end if;
                else  -- Take difference of cards to check who is closer to 21
                  PlayerDiff := "10101" - Optotal(i);
                  HPDiff     := "10101" - Optotal(n-1);
                  if (HPDiff > PlayerDiff) then
                    Action(2*i+1 downto 2*i) <= "10";
                  else
                    Action(2*i+1 downto 2*i) <= "00";
                  end if;
                end if;
              end if;
            end if;
          end loop;
          InitOptotal <= '1';
          TakeAction  <= '1';           -- Send signal to banker
        end if;
      end if;
    end if;
  end process;
end behavior;














