LIBRARY ieee;
USE ieee.std_logic_1164.ALL;
USE ieee.std_logic_arith.ALL;
USE ieee.std_logic_unsigned.ALL;
use work.controlFuncs.all;


entity top_TB is
    
end top_TB;

architecture structure of top_TB is
  CONSTANT nPlayers : INTEGER := 3;     -- Number of Players
  COMPONENT controller
    GENERIC (
      nPlayers : INTEGER);
    PORT (
      clock       : IN  STD_LOGIC;
      enable      : IN  STD_LOGIC;
      reset       : IN  STD_LOGIC;
      bistEnable  : IN  STD_LOGIC;
      pHit        : IN  STD_LOGIC_VECTOR(nPlayers-1 DOWNTO 0);
      pStand      : IN  STD_LOGIC_VECTOR(nPlayers-1 DOWNTO 0);
      issued      : IN  STD_LOGIC;
      gameOver    : IN  STD_LOGIC;
      balanceRdy  : IN  STD_LOGIC;
      skBust      : IN  STD_LOGIC;
      playerID    : OUT STD_LOGIC_VECTOR(nPlayers-1 DOWNTO 0);
      globalReset : OUT STD_LOGIC;
      cgIssue     : OUT STD_LOGIC;
      cgNewHand   : OUT STD_LOGIC;
      cgBist      : OUT STD_LOGIC;
      handOver    : OUT STD_LOGIC;
      pbr  : out STD_LOGIC);
  END COMPONENT;

  component Bjsk
    generic (
      n : integer);
    port (
      Clk        : in  std_logic;
      Reset      : in  std_logic;
      Card       : in  std_logic_vector (3 downto 0);
      PlayerId   : in  std_logic_vector (n-1 downto 0);
      cgIssue    : in  std_logic;
      PlStand    : in  std_logic;
      GameOver   : in  std_logic;
      Action     : out std_logic_vector (2*n-3 downto 0);
      TakeAction : out std_logic;
      skBust     : out std_logic);
  end component;
  
  component banker
    generic (
      n : integer);
    port (
      Clk        : IN  STD_LOGIC;
      Reset      : IN  STD_LOGIC;
      playerId   : IN  STD_LOGIC_VECTOR (n-1 downto 0);
      Action     : IN  STD_LOGIC_VECTOR (2*n-3 downto 0);
      TakeAction : IN  STD_LOGIC;
      cgNewHand  : IN  STD_LOGIC;
      bet        : IN  STD_LOGIC_VECTOR (11 DOWNTO 0);
      balance    : OUT STD_LOGIC_VECTOR (11 downto 0);
      GameOver   : OUT STD_LOGIC;
      balancerdy : OUT STD_LOGIC);
  end component;

  COMPONENT card_generator
    PORT (
      in_clock      : in  std_logic;
      in_reset      : in  std_logic;
      in_issue      : in  std_logic;
      in_bist       : in  std_logic;
      in_num_twos   : in  std_logic_vector(4 downto 0);
      in_num_threes : in  std_logic_vector(4 downto 0);
      in_num_fours  : in  std_logic_vector(4 downto 0);
      in_num_fives  : in  std_logic_vector(4 downto 0);
      in_num_sixes  : in  std_logic_vector(4 downto 0);
      in_num_sevens : in  std_logic_vector(4 downto 0);
      in_num_eights : in  std_logic_vector(4 downto 0);
      in_num_nines  : in  std_logic_vector(4 downto 0);
      in_num_tens   : in  std_logic_vector(4 downto 0);
      in_num_jacks  : in  std_logic_vector(4 downto 0);
      in_num_kings  : in  std_logic_vector(4 downto 0);
      in_num_queens : in  std_logic_vector(4 downto 0);
      in_num_aces   : in  std_logic_vector(4 downto 0);
      out_card      : out std_logic_vector(3 downto 0);
      out_house     : out std_logic_vector(3 downto 0);
      out_issued    : out std_logic);
  END COMPONENT;

  COMPONENT cardcounter
    PORT (
      clock      : IN  STD_LOGIC;
      reset      : IN  STD_LOGIC;
      issued     : IN  STD_LOGIC;
      card       : IN  STD_LOGIC_VECTOR (3 downto 0);
      stat_ace   : OUT STD_LOGIC_VECTOR (4 downto 0);
      stat_two   : OUT STD_LOGIC_VECTOR (4 downto 0);
      stat_three : OUT STD_LOGIC_VECTOR (4 downto 0);
      stat_four  : OUT STD_LOGIC_VECTOR (4 downto 0);
      stat_five  : OUT STD_LOGIC_VECTOR (4 downto 0);
      stat_six   : OUT STD_LOGIC_VECTOR (4 downto 0);
      stat_seven : OUT STD_LOGIC_VECTOR (4 downto 0);
      stat_eight : OUT STD_LOGIC_VECTOR (4 downto 0);
      stat_nine  : OUT STD_LOGIC_VECTOR (4 downto 0);
      stat_ten   : OUT STD_LOGIC_VECTOR (4 downto 0);
      stat_jack  : OUT STD_LOGIC_VECTOR (4 downto 0);
      stat_queen : OUT STD_LOGIC_VECTOR (4 downto 0);
      stat_king  : OUT STD_LOGIC_VECTOR (4 downto 0));
  END COMPONENT;

  COMPONENT housePlayer
    PORT (
      clock                         : IN  STD_LOGIC;
      reset                         : IN  STD_LOGIC;
      turn                          : IN  STD_LOGIC;
      issued                        : IN  STD_LOGIC;
      card                          : IN  STD_LOGIC_VECTOR(3 DOWNTO 0);
      dealerShow                    : IN  STD_LOGIC_VECTOR(3 DOWNTO 0);
      NumA, Num2, Num3, Num4, Num5  : IN  STD_LOGIC_VECTOR(4 DOWNTO 0);
      Num6, Num7, Num8, Num9, Num10 : IN  STD_LOGIC_VECTOR(4 DOWNTO 0);
      NumJ, NumQ, NumK              : IN  STD_LOGIC_VECTOR(4 DOWNTO 0);
      numHands                      : IN  STD_LOGIC_VECTOR(6 DOWNTO 0);
      balance                       : IN  STD_LOGIC_VECTOR(9 DOWNTO 0);
      balanceRdy                    : in  STD_LOGIC;
      hit                           : OUT STD_LOGIC;
      stand                         : OUT STD_LOGIC;
      bet                           : OUT STD_LOGIC_VECTOR(9 DOWNTO 0));
  END COMPONENT;

  COMPONENT basicPlayer
    PORT (
      clock                         : IN  STD_LOGIC;
      reset                         : IN  STD_LOGIC;
      turn                          : IN  STD_LOGIC;
      issued                        : IN  STD_LOGIC;
      card                          : IN  STD_LOGIC_VECTOR(3 DOWNTO 0);
      dealerShow                    : IN  STD_LOGIC_VECTOR(3 DOWNTO 0);
      NumA, Num2, Num3, Num4, Num5  : IN  STD_LOGIC_VECTOR(4 DOWNTO 0);
      Num6, Num7, Num8, Num9, Num10 : IN  STD_LOGIC_VECTOR(4 DOWNTO 0);
      NumJ, NumQ, NumK              : IN  STD_LOGIC_VECTOR(4 DOWNTO 0);
      numHands                      : IN  STD_LOGIC_VECTOR(6 DOWNTO 0);
      balance                       : IN  STD_LOGIC_VECTOR(9 DOWNTO 0);
      balanceRdy                    : in  STD_LOGIC;
      hit                           : OUT STD_LOGIC;
      stand                         : OUT STD_LOGIC;
      bet                           : OUT STD_LOGIC_VECTOR(9 DOWNTO 0));
  END COMPONENT;

  COMPONENT fileBasicPlayer
    PORT (
      clock                         : IN  STD_LOGIC;
      reset                         : IN  STD_LOGIC;
      turn                          : IN  STD_LOGIC;
      issued                        : IN  STD_LOGIC;
      card                          : IN  STD_LOGIC_VECTOR(3 DOWNTO 0);
      dealerShow                    : IN  STD_LOGIC_VECTOR(3 DOWNTO 0);
      NumA, Num2, Num3, Num4, Num5  : IN  STD_LOGIC_VECTOR(4 DOWNTO 0);
      Num6, Num7, Num8, Num9, Num10 : IN  STD_LOGIC_VECTOR(4 DOWNTO 0);
      NumJ, NumQ, NumK              : IN  STD_LOGIC_VECTOR(4 DOWNTO 0);
      numHands                      : IN  STD_LOGIC_VECTOR(6 DOWNTO 0);
      balance                       : IN  STD_LOGIC_VECTOR(9 DOWNTO 0);
      balanceRdy                    : in  STD_LOGIC;
      hit                           : OUT STD_LOGIC;
      stand                         : OUT STD_LOGIC;
      bet                           : OUT STD_LOGIC_VECTOR(9 DOWNTO 0));
  END COMPONENT;


  signal Action     : STD_LOGIC_VECTOR (2*nPlayers-3 downto 0);
  signal TakeAction : STD_LOGIC;

  
  SIGNAL NumA   : STD_LOGIC_VECTOR (4 downto 0);
  SIGNAL Num2   : STD_LOGIC_VECTOR (4 downto 0);
  SIGNAL Num3 : STD_LOGIC_VECTOR (4 downto 0);
  SIGNAL Num4  : STD_LOGIC_VECTOR (4 downto 0);
  SIGNAL Num5  : STD_LOGIC_VECTOR (4 downto 0);
  SIGNAL Num6   : STD_LOGIC_VECTOR (4 downto 0);
  SIGNAL Num7 : STD_LOGIC_VECTOR (4 downto 0);
  SIGNAL Num8 : STD_LOGIC_VECTOR (4 downto 0);
  SIGNAL Num9  : STD_LOGIC_VECTOR (4 downto 0);
  SIGNAL Num10   : STD_LOGIC_VECTOR (4 downto 0);
  SIGNAL NumJ  : STD_LOGIC_VECTOR (4 downto 0);
  SIGNAL NumQ : STD_LOGIC_VECTOR (4 downto 0);
  SIGNAL NumK  : STD_LOGIC_VECTOR (4 downto 0);


  SIGNAL card        : std_logic_vector(3 downto 0);
  SIGNAL dealerShow  : std_logic_vector(3 downto 0);
  SIGNAL issued      : std_logic;

  SIGNAL clock       : STD_LOGIC;
  SIGNAL enable      : STD_LOGIC;
  SIGNAL reset       : STD_LOGIC;
  SIGNAL bistEnable  : STD_LOGIC;
  SIGNAL pHit        : STD_LOGIC_VECTOR(nPlayers-1 DOWNTO 0);
  SIGNAL pStand      : STD_LOGIC_VECTOR(nPlayers-1 DOWNTO 0);
  SIGNAL gameOver    : STD_LOGIC;
  SIGNAL balanceRdy  : STD_LOGIC;
  signal bankerBalanceRdy : std_logic;
  SIGNAL skBust      : STD_LOGIC;
  SIGNAL playerID    : STD_LOGIC_VECTOR(nPlayers-1 DOWNTO 0);
  SIGNAL globalReset : STD_LOGIC;
  SIGNAL cgIssue     : STD_LOGIC;
  SIGNAL cgNewHand   : STD_LOGIC;
  SIGNAL cgBist      : STD_LOGIC;
  SIGNAL handOver    : STD_LOGIC;


  SIGNAL numHands                      : STD_LOGIC_VECTOR(6 DOWNTO 0);
  SIGNAL balance                       : STD_LOGIC_VECTOR(9 DOWNTO 0);
  SIGNAL bet                           : STD_LOGIC_VECTOR(9 DOWNTO 0);

  SIGNAL cgCARD : STD_LOGIC_VECTOR(3 DOWNTO 0);
begin  -- structure

  banker_1: banker
    generic map (
      n => nPlayers)
    port map (
      Clk        => clock,
      Reset      => globalReset,
      playerId   => playerId,
      Action     => Action,
      TakeAction => TakeAction,
      cgNewHand  => cgNewHand,
      bet(9 downto 0)        => bet,
      bet(11 downto 10) => "00",
      balance(9 downto 0)    => balance,
      balance(11 downto 10) => "00",
      GameOver   => gameOver,
      balancerdy => bankerBalanceRdy);

  Bjsk_1: Bjsk
    generic map (
      n => nPlayers)
    port map (
      Clk        => clock,
--      Reset      => globalReset,
      Reset      => cgNewHand,
      Card       => card,
      PlayerId   => PlayerId,
      cgIssue    => cgIssue,
      PlStand    => handOver,            -- What is this?
      GameOver   => globalReset,
      Action     => Action,
      TakeAction => TakeAction,
      skBust     => skBust);


  
  controller_1: controller
    GENERIC MAP (
      nPlayers => nPlayers)
    PORT MAP (
      clock       => clock,
      enable      => enable,
      reset       => reset,
      bistEnable  => bistEnable,
      pHit        => pHit,
      pStand      => pStand,
      issued      => issued,
      gameOver    => gameOver,
      balanceRdy  => bankerBalanceRdy,
      skBust      => skBust,
      playerID    => playerID,
      globalReset => globalReset,
      cgIssue     => cgIssue,
      cgNewHand   => cgNewHand,
      cgBist      => cgBist,
      handOver    => handOver,
      pbr => balanceRdy);

  cardcounter_1: cardcounter
    PORT MAP (
      clock      => clock,
      reset      => cgNewHand,
      issued     => issued,
      card       => card,
      stat_ace   => NumA,
      stat_two   => Num2,
      stat_three => Num3,
      stat_four  => Num4,
      stat_five  => Num5,
      stat_six   => Num6,
      stat_seven => Num7,
      stat_eight => Num8,
      stat_nine  => Num9,
      stat_ten   => Num10,
      stat_jack  => NumJ,
      stat_queen => NumQ,
      stat_king  => NumK);

  
  card_generator_1: card_generator
    PORT MAP (
      in_clock      => clock,
      in_reset      => cgNewHand,
      in_issue      => cgIssue,
      in_bist       => cgBist,
      in_num_twos   => Num2,
      in_num_threes => Num3,
      in_num_fours  => Num4,
      in_num_fives  => Num5,
      in_num_sixes  => Num6,
      in_num_sevens => Num7,
      in_num_eights => Num8,
      in_num_nines  => Num9,
      in_num_tens   => Num10,
      in_num_jacks  => NumJ,
      in_num_kings  => NumK,
      in_num_queens => NumQ,
      in_num_aces   => NumA,
      out_card      => card,
      out_house     => dealerShow,
      out_issued    => issued);

  housePlayer_1: housePlayer
    PORT MAP (
      clock      => clock,
      reset      => reset,
      turn       => playerID(nPlayers-1),
      issued     => issued,
      card       => card,
      dealerShow => dealerShow,
      NumA       => NumA,
      Num2       => Num2,
      Num3       => Num3,
      Num4       => Num4,
      Num5       => Num5,
      Num6       => Num6,
      Num7       => Num7,
      Num8       => Num8,
      Num9       => Num9,
      Num10      => Num10,
      NumJ       => NumJ,
      NumQ       => NumQ,
      NumK       => NumK,
      numHands   => numHands,
      balance    => balance,
      balanceRdy => balanceRdy,
      hit        => pHit(nPlayers-1),
      stand      => pStand(nPlayers-1),
      bet        => bet);

  basicPlayer_1: basicPlayer
    PORT MAP (
      clock      => clock,
      reset      => reset,
      turn       => playerID(0),
      issued     => issued,
      card       => card,
      dealerShow => dealerShow,
      NumA       => NumA,
      Num2       => Num2,
      Num3       => Num3,
      Num4       => Num4,
      Num5       => Num5,
      Num6       => Num6,
      Num7       => Num7,
      Num8       => Num8,
      Num9       => Num9,
      Num10      => Num10,
      NumJ       => NumJ,
      NumQ       => NumQ,
      NumK       => NumK,
      numHands   => numHands,
      balance    => balance,
      balanceRdy => balanceRdy,
      hit        => pHit(0),
      stand      => pStand(0),
      bet        => bet);


  basicPlayer_2: fileBasicPlayer
    PORT MAP (
      clock      => clock,
      reset      => reset,
      turn       => playerID(1),
      issued     => issued,
      card       => card,
      dealerShow => dealerShow,
      NumA       => NumA,
      Num2       => Num2,
      Num3       => Num3,
      Num4       => Num4,
      Num5       => Num5,
      Num6       => Num6,
      Num7       => Num7,
      Num8       => Num8,
      Num9       => Num9,
      Num10      => Num10,
      NumJ       => NumJ,
      NumQ       => NumQ,
      NumK       => NumK,
      numHands   => numHands,
      balance    => balance,
      balanceRdy => balanceRdy,
      hit        => pHit(1),
      stand      => pStand(1),
      bet        => bet);
  

init_proc: PROCESS
BEGIN  -- PROCESS init_proc
  reset <= '1';

  WAIT FOR 80 ns;
  reset <= '0';


  -- ScoreKeeper should do this after the hands are played
  wait until gameOver = '1';
  report "gameOver state reached";
  wait;
--  reset <= '1';

END PROCESS init_proc;


--fake_bank : PROCESS (playerID)
--BEGIN
--  IF playerID(0) = '1' THEN
--    
--  END IF;
--  
--  IF handOver = '1' THEN
--    balanceRdy <= '1';
--  ELSE
--    balanceRdy <= '0';
--  END IF;
--
--END process;
  

clock_proc : PROCESS
begin
  --this process was generated based on formula: 0 0 ns, 1 20 ns -r 40 ns
  --wait for <time to next event>; -- <current time>
  clock <= '0';
  wait for 20 ns; --0 fs
  
  clock <= '1';
  wait for 20 ns; --20 ns
end process;
  
  
end structure;
